//
// WiFi to  WiFi test
//

#define DEBUG
#ifdef DEBUG
#define DebugPort Serial    // Programming port
#define DBPrint(x)   if(DebugPort) DebugPort.print(x)
#define DBPrintln(x) if(DebugPort) DebugPort.println(x)
#else
#define DBPrint(x)
#define DBPrintln(x)
#endif // DEBUG

#include "Arduino.h"
#include <atomic>

String IpAddress2String(const IPAddress& ipAddress)
{
  return String(ipAddress[0]) + String(".") +
  		String(ipAddress[1]) + String(".") +
		String(ipAddress[2]) + String(".") +
		String(ipAddress[3]); 
}
#define WIFI_VAR_LEN 64
typedef struct WIFICONFIG {
	IPAddress       ip;
	char 			sSSID[WIFI_VAR_LEN];
	char			sPassword[WIFI_VAR_LEN];
} WIFIConfig;

#include <WiFi.h>
#define SHUTTER_PORT 2424
WiFiMulti shutterWiFi;
WIFIConfig wifiConfig;
std::atomic<bool> wifiPresent = false;
std::atomic<bool> core0Ready = false;
bool configureWiFi();
bool initWiFi(IPAddress ip, String sSSID, String sPassword);

void setup()
{
	core0Ready = false;
#ifdef DEBUG
	DebugPort.begin(115200);
	delay(1000);
	DBPrintln("========== RTI-Zone controller booting ==========");
#endif

	wifiConfig.ip.fromString("172.31.255.2");
	strncpy(wifiConfig.sSSID,"RTIShutter", WIFI_VAR_LEN);
	strncpy(wifiConfig.sPassword,"RTIShutter", WIFI_VAR_LEN);

	WiFi.mode(WIFI_STA);
	WiFi.setHostname("RTI-Shutter");
	wifiPresent = configureWiFi();
	core0Ready = true;
	DBPrintln("========== Core 0 ready ==========");
}

void setup1()
{
	while(!core0Ready)
		delay(100);
	DBPrintln("========== Core 1 ready ==========");
}


void loop()
{
	if(wifiPresent) {
		PingRotator();
	}
	else {
		wifiPresent = configureWiFi();
	}
	delay(1000); // ping once a second
}

//
// This loop does all the motor controls
//
void loop1()
{   
	// in the real code this runs some code to control a stepper motor via AccelStepper
	delay(1000);
}

// WiFi connection to rotator
bool configureWiFi()
{
	DBPrintln("========== Configuring WiFi ==========");

	return initWiFi(wifiConfig.ip,
			String(wifiConfig.sSSID),
			String(wifiConfig.sPassword));
}

bool initWiFi(IPAddress ip, String sSSID, String sPassword)
{
	WiFi.config(ip);
	shutterWiFi.addAP(sSSID.c_str(), sPassword.c_str());
	if(shutterWiFi.run()!=WL_CONNECTED) {
		DBPrintln("========== Failed to start WiFi AP ==========");
		return false;
	}
	DBPrintln("IP = " + IpAddress2String(WiFi.localIP()));
	WiFi.setTimeout(25);
	return true;
}


void PingRotator()
{
	int nRes = 0;
	nRes = WiFi.ping("172.31.255.1");
	DBPrintln("WiFi 172.31.255.1 ping result : " + String(nRes>0?"Ok":"Error"));
	if(nRes<0) {
		DBPrintln("WiFi client disconnected");
		wifiPresent = false;
	}
}

